import { ArrCls, BOOLEAN, FUNCTION, NUMBER, OBJECT, ObjProto, STRING, UNDEFINED, UNDEF_VALUE } from "../internal/constants";
import { _safeCheck } from "../internal/safe_check";
export function _createIs(theType) {
    return function (value) {
        return typeof value === theType;
    };
}
export function _createObjIs(theType) {
    return function (value) {
        return !!(value && objToString(value) === theType);
    };
}
export function objToString(value) {
    return ObjProto.toString.call(value);
}
export function isTypeof(value, theType) {
    return typeof value === theType;
}
export function isUndefined(value) {
    return typeof value === UNDEFINED || value === UNDEFINED;
}
export function isStrictUndefined(arg) {
    return !isDefined(arg);
}
export function isNullOrUndefined(value) {
    return value === null || isUndefined(value);
}
export function isStrictNullOrUndefined(value) {
    return value === null || !isDefined(value);
}
export function isDefined(arg) {
    return !!arg || arg !== UNDEF_VALUE;
}
export var isString = _createIs(STRING);
export var isFunction = _createIs(FUNCTION);
export function isObject(value) {
    if (isNullOrUndefined(value)) {
        return false;
    }
    return !!value && typeof value === OBJECT;
}
export var isArray = ArrCls.isArray;
export var isDate = _createObjIs("[object Date]");
export var isNumber = _createIs(NUMBER);
export var isBoolean = _createIs(BOOLEAN);
export var isRegExp = _createObjIs("[object RegExp]");
export var isFile = _createObjIs("[object File]");
export var isFormData = _createObjIs("[object FormData]");
export var isBlob = _createObjIs("[object Blob]");
export var isArrayBuffer = _createObjIs("[object ArrayBuffer]");
export var isError = _createObjIs("[object Error]");
export function isPromiseLike(value) {
    return !!value && isFunction(value.then);
}
export function isPromise(value) {
    return isPromiseLike(value) && isFunction(value.catch);
}
export function isNotTruthy(value) {
    return !value || !_safeCheck(function () { return (value && (0 + value)); }, value);
}
export function isTruthy(value) {
    return !(!value || _safeCheck(function () { return !(value && (0 + value)); }, !value));
}
//# sourceMappingURL=base.js.map