import { ObjClass, VALUE } from "../internal/constants";
import { isFunction, isUndefined, objToString } from "../helpers/base";
import { throwUnsupported } from "../helpers/customError";
import { dumpObj } from "../helpers/diagnostics";
var _objDefineProperty = ObjClass["defineProperty"];
export function objDefineProp(target, key, descriptor) {
    if (_objDefineProperty) {
        _objDefineProperty(target, key, descriptor);
        return target;
    }
    throwUnsupported("Unable to define property [" + objToString(key) + "] on " + dumpObj(target));
}
export function objDefineGet(target, key, value, configurable) {
    var desc = {
        enumerable: true,
        configurable: isUndefined(configurable) ? true : configurable
    };
    if (isFunction(value)) {
        desc.get = value;
    }
    else {
        desc[VALUE] = value;
    }
    return objDefineProp(target, key, desc);
}
export function objDefineAccessors(target, prop, getProp, setProp, configurable) {
    if (_objDefineProperty) {
        try {
            var descriptor = {
                enumerable: true,
                configurable: isUndefined(configurable) ? true : configurable
            };
            if (getProp) {
                descriptor.get = getProp;
            }
            if (setProp) {
                descriptor.set = setProp;
            }
            return objDefineProp(target, prop, descriptor);
        }
        catch (e) {
            throwUnsupported("Unable to define accessors for [" + objToString(prop) + "] on " + dumpObj(target));
        }
    }
}
//# sourceMappingURL=define.js.map