import { isNullOrUndefined } from "../helpers/base";
import { getGlobal } from "../helpers/environment";
import { _wellKnownSymbolMap } from "../symbol/well_known";
import { throwTypeError } from "../helpers/throw";
import { POLYFILL_TAG, SYMBOL } from "../internal/constants";
import { objHasOwnProperty } from "../object/has_own_prop";
var POLY_GLOBAL_REGISTORT = "__polySymbols$ts_utils";
var _polySymbols;
function _globalSymbolRegistry() {
    if (!_polySymbols) {
        var gbl = getGlobal();
        _polySymbols = gbl[POLY_GLOBAL_REGISTORT] = gbl[POLY_GLOBAL_REGISTORT] || { k: {}, s: {} };
    }
    return _polySymbols;
}
var _wellKnownSymbolCache = {};
export function polyNewSymbol(description) {
    var theSymbol = {
        description: "" + description,
        toString: function () { return SYMBOL + "(" + description + ")"; }
    };
    theSymbol[POLYFILL_TAG] = true;
    return theSymbol;
}
export function polySymbolFor(key) {
    var registry = _globalSymbolRegistry();
    if (!objHasOwnProperty(registry, key)) {
        var newSymbol = polyNewSymbol(key);
        registry.k[key] = newSymbol;
        registry.s[newSymbol] = "" + key;
    }
    return registry.k[key];
}
export function polySymbolKeyFor(sym) {
    if (isNullOrUndefined(sym) || sym == "" + null) {
        throwTypeError(sym + " is not a symbol");
    }
    return _globalSymbolRegistry().s[sym];
}
export function polyGetKnownSymbol(name) {
    var result;
    var knownName = _wellKnownSymbolMap[name];
    if (knownName) {
        result = _wellKnownSymbolCache[knownName] = _wellKnownSymbolCache[knownName] || polyNewSymbol(SYMBOL + "." + knownName);
    }
    return result;
}
//# sourceMappingURL=symbol.js.map