import { isNullOrUndefined, isString } from "../helpers/base";
import { dumpObj } from "../helpers/diagnostics";
import { throwRangeError, throwTypeError } from "../helpers/throw";
import { EMPTY, StrProto } from "../internal/constants";
import { _unwrapFunction } from "../internal/unwrapFunction";
import { mathToInt } from "../math/to_int";
var REPEAT = "repeat";
export var strRepeat = StrProto[REPEAT] ? _unwrapFunction(REPEAT) : polyStrRepeat;
export function polyStrRepeat(value, count) {
    if (isNullOrUndefined(value)) {
        throwTypeError("can't convert [" + dumpObj(value) + "]");
    }
    count = mathToInt(count, true);
    if (count < 0) {
        throwRangeError("invalid count must be >= 0 && < Infinity");
    }
    var pad = isString(value) ? value : "" + value;
    var result = EMPTY;
    for (; count > 0; (count >>>= 1) && (pad += pad)) {
        if (count & 1) {
            result += pad;
        }
    }
    return result;
}
//# sourceMappingURL=repeat.js.map