/*!
 * NevWare21 - ts-utils, 0.4.2
 * https://github.com/nevware21/ts-utils
 * Copyright (c) NevWare21 and contributors. All rights reserved.
 * Licensed under the MIT license.
 */
System.register('nevware21.ts-utils', [], (function (exports) {
    'use strict';
    return {
        execute: (function () {

            exports({
                arrAppend: arrAppend,
                arrForEach: arrForEach,
                createArrayIterator: createArrayIterator,
                createCustomError: createCustomError,
                createEnum: createEnum,
                createEnumKeyMap: createEnumKeyMap,
                createEnumValueMap: createEnumValueMap,
                createIterable: createIterable,
                createIterator: createIterator,
                createRangeIterator: createRangeIterator,
                createSimpleMap: createSimpleMap,
                createTypeMap: createTypeMap,
                deepExtend: deepExtend,
                dumpObj: dumpObj,
                getDocument: getDocument,
                getGlobal: getGlobal,
                getHistory: getHistory,
                getInst: getInst,
                getIntValue: getIntValue,
                getKnownSymbol: getKnownSymbol,
                getNavigator: getNavigator,
                getSymbol: getSymbol,
                getWindow: getWindow,
                hasDocument: hasDocument,
                hasHistory: hasHistory,
                hasNavigator: hasNavigator,
                hasSymbol: hasSymbol,
                hasValue: hasValue,
                hasWindow: hasWindow,
                isDefined: isDefined,
                isIterable: isIterable,
                isIterator: isIterator,
                isNode: isNode,
                isNotTruthy: isNotTruthy,
                isNullOrUndefined: isNullOrUndefined,
                isObject: isObject,
                isPromise: isPromise,
                isPromiseLike: isPromiseLike,
                isStrictNullOrUndefined: isStrictNullOrUndefined,
                isStrictUndefined: isStrictUndefined,
                isTruthy: isTruthy,
                isTypeof: isTypeof,
                isUndefined: isUndefined,
                isWebWorker: isWebWorker,
                iterForOf: iterForOf,
                makeIterable: makeIterable,
                mathToInt: mathToInt,
                newSymbol: newSymbol,
                objCopyProps: objCopyProps,
                objDeepCopy: objDeepCopy,
                objDeepFreeze: objDeepFreeze,
                objDefineAccessors: objDefineAccessors,
                objDefineGet: objDefineGet,
                objDefineProp: objDefineProp,
                objExtend: objExtend,
                objForEachKey: objForEachKey,
                objHasOwnProperty: objHasOwnProperty,
                objKeys: objKeys,
                objSetPrototypeOf: objSetPrototypeOf,
                objToString: objToString,
                polyGetKnownSymbol: polyGetKnownSymbol,
                polyIsArray: polyIsArray,
                polyNewSymbol: polyNewSymbol,
                polyObjKeys: polyObjKeys,
                polyStrSubstr: polyStrSubstr,
                polySymbolFor: polySymbolFor,
                polySymbolKeyFor: polySymbolKeyFor,
                strIsNullOrEmpty: strIsNullOrEmpty,
                strIsNullOrWhiteSpace: strIsNullOrWhiteSpace,
                strLeft: strLeft,
                strRight: strRight,
                throwError: throwError,
                throwRangeError: throwRangeError,
                throwTypeError: throwTypeError,
                throwUnsupported: throwUnsupported
            });

            var UNDEF_VALUE = undefined;
            var EMPTY = "";
            var BOOLEAN = "boolean";
            var FUNCTION = "function";
            var NUMBER = "number";
            var OBJECT = "object";
            var PROTOTYPE = "prototype";
            var STRING = "string";
            var UNDEFINED = "undefined";
            var HAS_OWN_PROPERTY = "hasOwnProperty";
            var SYMBOL = "Symbol";
            var POLYFILL_TAG = "_polyfill";
            var INDEX_OF = "indexOf";
            var LAST_INDEX_OF = "lastIndexOf";
            var LENGTH = "length";
            var DONE = "done";
            var VALUE = "value";
            var ObjClass = Object;
            var ObjProto = ObjClass[PROTOTYPE];
            var StrCls = String;
            var StrProto = StrCls[PROTOTYPE];
            var MathCls = Math;
            var ArrCls = Array;
            var ArrProto = ArrCls[PROTOTYPE];

            function _safeCheck(cb, defValue) {
                var result = defValue;
                try {
                    result = cb();
                }
                catch (e) {
                }
                return result;
            }

            function _createIs(theType) {
                return function (value) {
                    return typeof value === theType;
                };
            }
            function _createObjIs(theType) {
                return function (value) {
                    return !!(value && objToString(value) === theType);
                };
            }
            function objToString(value) {
                return ObjProto.toString.call(value);
            }
            function isTypeof(value, theType) {
                return typeof value === theType;
            }
            function isUndefined(value) {
                return typeof value === UNDEFINED || value === UNDEFINED;
            }
            function isStrictUndefined(arg) {
                return !isDefined(arg);
            }
            function isNullOrUndefined(value) {
                return value === null || isUndefined(value);
            }
            function isStrictNullOrUndefined(value) {
                return value === null || !isDefined(value);
            }
            function isDefined(arg) {
                return !!arg || arg !== UNDEF_VALUE;
            }
            var isString = exports('isString', _createIs(STRING));
            var isFunction = exports('isFunction', _createIs(FUNCTION));
            function isObject(value) {
                if (isNullOrUndefined(value)) {
                    return false;
                }
                return !!value && typeof value === OBJECT;
            }
            var isArray = exports('isArray', ArrCls.isArray);
            var isDate = exports('isDate', _createObjIs("[object Date]"));
            var isNumber = exports('isNumber', _createIs(NUMBER));
            var isBoolean = exports('isBoolean', _createIs(BOOLEAN));
            var isRegExp = exports('isRegExp', _createObjIs("[object RegExp]"));
            var isFile = exports('isFile', _createObjIs("[object File]"));
            var isFormData = exports('isFormData', _createObjIs("[object FormData]"));
            var isBlob = exports('isBlob', _createObjIs("[object Blob]"));
            var isArrayBuffer = exports('isArrayBuffer', _createObjIs("[object ArrayBuffer]"));
            var isError = exports('isError', _createObjIs("[object Error]"));
            function isPromiseLike(value) {
                return !!value && isFunction(value.then);
            }
            function isPromise(value) {
                return isPromiseLike(value) && isFunction(value.catch);
            }
            function isNotTruthy(value) {
                return !value || !_safeCheck(function () { return (value && (0 + value)); }, value);
            }
            function isTruthy(value) {
                return !(!value || _safeCheck(function () { return !(value && (0 + value)); }, !value));
            }

            var DOCUMENT = "document";
            var HISTORY = "history";
            var NAVIGATOR = "navigator";
            var WINDOW = "window";
            var _cachedGlobal = null;
            var _hasWindow = _safeCheck(function () { return isDefined(window); }, false);
            var _hasDocument = _safeCheck(function () { return isDefined(document); }, false);
            var _hasNavigator = _safeCheck(function () { return isDefined(navigator); }, false);
            var _hasHistory = _safeCheck(function () { return isDefined(history); }, false);
            var _isWebWorker = _safeCheck(function () { return !!(self && self instanceof WorkerGlobalScope); }, false);
            var _isNode = _safeCheck(function () { return !!(process && (process.versions || {}).node); }, false);
            function getGlobal(useCached) {
                var result = useCached === false ? null : _cachedGlobal;
                if (!result) {
                    if (!result && typeof globalThis !== UNDEFINED) {
                        result = globalThis;
                    }
                    if (!result && typeof self !== UNDEFINED) {
                        result = self;
                    }
                    if (!result && typeof window !== UNDEFINED) {
                        result = window;
                    }
                    if (!result && typeof global !== UNDEFINED) {
                        result = global;
                    }
                    _cachedGlobal = result;
                }
                return result;
            }
            function getInst(name, useCached) {
                var gbl = getGlobal(useCached);
                if (gbl && gbl[name]) {
                    return gbl[name];
                }
                if (name === WINDOW && _hasWindow) {
                    return window;
                }
                return null;
            }
            function hasDocument() {
                return !!getDocument();
            }
            function getDocument() {
                return _hasDocument ? document : getInst(DOCUMENT);
            }
            function hasWindow() {
                return !!getWindow();
            }
            function getWindow() {
                return _hasWindow ? window : getInst(WINDOW);
            }
            function hasNavigator() {
                return !!getNavigator();
            }
            function getNavigator() {
                return _hasNavigator ? navigator : getInst(NAVIGATOR);
            }
            function hasHistory() {
                return !!getHistory();
            }
            function getHistory() {
                return _hasHistory ? history : getInst(HISTORY);
            }
            function isNode() {
                return _isNode;
            }
            function isWebWorker() {
                return _isWebWorker;
            }

            function objHasOwnProperty(obj, prop) {
                return obj && ObjProto[HAS_OWN_PROPERTY].call(obj, prop);
            }

            function objForEachKey(theObject, callbackfn, thisArg) {
                if (theObject && isObject(theObject)) {
                    for (var prop in theObject) {
                        if (objHasOwnProperty(theObject, prop)) {
                            if (callbackfn.call(thisArg || theObject, prop, theObject[prop]) === -1) {
                                break;
                            }
                        }
                    }
                }
            }

            function arrForEach(arr, callbackfn, thisArg) {
                if (arr) {
                    var len = arr[LENGTH];
                    for (var idx = 0; idx < len; idx++) {
                        if (idx in arr) {
                            if (callbackfn.call(thisArg || arr, arr[idx], idx, arr) === -1) {
                                break;
                            }
                        }
                    }
                }
            }

            function throwError(message) {
                throw new Error(message);
            }
            function throwTypeError(message) {
                throw new TypeError(message);
            }
            function throwRangeError(message) {
                throw new RangeError(message);
            }

            var ObjAssign = ObjClass["assign"];
            var _objFreeze = ObjClass["freeze"];
            function _doNothing(value) {
                return value;
            }
            var objAssign = exports('objAssign', ObjAssign);
            function objKeys(value) {
                if (!isObject(value) || value === null) {
                    throwTypeError("objKeys called on non-object");
                }
                return ObjClass.keys(value);
            }
            function objDeepCopy(source) {
                if (isArray(source)) {
                    var result_1 = [];
                    arrForEach(source, function (value) {
                        result_1.push(objDeepCopy(value));
                    });
                    return result_1;
                }
                if (isObject(source)) {
                    return objCopyProps({}, source);
                }
                return source;
            }
            function objCopyProps(target, source) {
                if (!isNullOrUndefined(source)) {
                    objForEachKey(source, function (key, value) {
                        target[key] = objDeepCopy(value);
                    });
                }
                return target;
            }
            function objDeepFreeze(value) {
                if (_objFreeze) {
                    objForEachKey(value, function (key, value) {
                        if (isArray(value) || isObject(value)) {
                            _objFreeze(value);
                        }
                    });
                }
                return objFreeze(value);
            }
            var objFreeze = exports('objFreeze', _objFreeze || _doNothing);
            var objSeal = exports('objSeal', ObjClass["seal"] || _doNothing);

            function createEnum(values) {
                var theEnum = {};
                objForEachKey(values, function (field, value) {
                    theEnum[field] = value;
                    theEnum[value] = field;
                });
                return objDeepFreeze(theEnum);
            }
            function createEnumKeyMap(values) {
                var mapClass = {};
                objForEachKey(values, function (key, value) {
                    mapClass[key] = key;
                    mapClass[value] = key;
                });
                return objDeepFreeze(mapClass);
            }
            function createEnumValueMap(values) {
                var mapClass = {};
                objForEachKey(values, function (key, value) {
                    mapClass[key] = value;
                    mapClass[value] = value;
                });
                return objDeepFreeze(mapClass);
            }
            function createSimpleMap(values) {
                var mapClass = {};
                objForEachKey(values, function (field, value) {
                    mapClass[field] = value[1];
                    mapClass[value[0]] = value[1];
                });
                return objDeepFreeze(mapClass);
            }
            function createTypeMap(values) {
                return createSimpleMap(values);
            }

            var _wellKnownSymbolMap = createEnumKeyMap({
                asyncIterator: 0,
                hasInstance: 1,
                isConcatSpreadable: 2,
                iterator: 3,
                match: 4,
                matchAll: 5,
                replace: 6,
                search: 7,
                species: 8,
                split: 9,
                toPrimitive: 10,
                toStringTag: 11,
                unscopables: 12
            });

            var POLY_GLOBAL_REGISTORT = "__polySymbols$ts_utils";
            var _polySymbols;
            function _globalSymbolRegistry() {
                if (!_polySymbols) {
                    var gbl = getGlobal();
                    _polySymbols = gbl[POLY_GLOBAL_REGISTORT] = gbl[POLY_GLOBAL_REGISTORT] || { k: {}, s: {} };
                }
                return _polySymbols;
            }
            var _wellKnownSymbolCache = {};
            function polyNewSymbol(description) {
                var theSymbol = {
                    description: "" + description,
                    toString: function () { return SYMBOL + "(" + description + ")"; }
                };
                theSymbol[POLYFILL_TAG] = true;
                return theSymbol;
            }
            function polySymbolFor(key) {
                var registry = _globalSymbolRegistry();
                if (!objHasOwnProperty(registry, key)) {
                    var newSymbol = polyNewSymbol(key);
                    registry.k[key] = newSymbol;
                    registry.s[newSymbol] = "" + key;
                }
                return registry.k[key];
            }
            function polySymbolKeyFor(sym) {
                if (isNullOrUndefined(sym) || sym == "" + null) {
                    throwTypeError(sym + " is not a symbol");
                }
                return _globalSymbolRegistry().s[sym];
            }
            function polyGetKnownSymbol(name) {
                var result;
                var knownName = _wellKnownSymbolMap[name];
                if (knownName) {
                    result = _wellKnownSymbolCache[knownName] = _wellKnownSymbolCache[knownName] || polyNewSymbol(SYMBOL + "." + knownName);
                }
                return result;
            }

            var _hasSymbol = _safeCheck(function () { return isDefined(Symbol); }, false);
            var _symbol = getInst(SYMBOL);
            var _symbolFor = _symbol && _safeCheck(function () { return _symbol["for"]; }, null);
            var _symbolKeyFor = _symbol && _safeCheck(function () { return _symbol["keyFor"]; }, null);
            var isSymbol = exports('isSymbol', _createIs("symbol"));
            function hasSymbol(useCached) {
                return !!getSymbol(useCached);
            }
            function getSymbol(useCached) {
                _symbol = useCached === false ? getInst(SYMBOL, useCached) : _symbol;
                return _hasSymbol && _symbol;
            }
            function getKnownSymbol(name, noPoly) {
                var knownName = _wellKnownSymbolMap[name];
                return _symbol ? _symbol[knownName || name] : (!noPoly ? polyGetKnownSymbol(name) : null);
            }
            function newSymbol(description, noPoly) {
                return _hasSymbol ? Symbol(description) : (!noPoly ? polyNewSymbol(description) : null);
            }
            var symbolFor = exports('symbolFor', _symbolFor || polySymbolFor);
            var symbolKeyFor = exports('symbolKeyFor', _symbolKeyFor || polySymbolKeyFor);

            function isIterator(value) {
                return !!value && isFunction(value.next);
            }
            function isIterable(value) {
                return !isStrictNullOrUndefined(value) && isFunction(value[getKnownSymbol(3)]);
            }

            function arrAppend(target, elms) {
                if (!isUndefined(elms) && target) {
                    if (isArray(elms)) {
                        ArrProto.push.apply(target, elms);
                    }
                    else if (isIterator(elms)) {
                        var value = elms.next();
                        while (!value[DONE]) {
                            target.push(value[VALUE]);
                            value = elms.next();
                        }
                    }
                    else {
                        target.push(elms);
                    }
                }
                return target;
            }

            function dumpObj(object, format) {
                var objectTypeDump = objToString(object);
                var propertyValueDump = "";
                if (objectTypeDump === "[object Error]") {
                    propertyValueDump = "{ stack: '" + object.stack + "', message: '" + object.message + "', name: '" + object.name + "'";
                }
                else {
                    if (format) {
                        if (isNumber(format)) {
                            propertyValueDump = JSON.stringify(object, null, format);
                        }
                        else {
                            propertyValueDump = JSON.stringify(object, null, format ? 4 : 0);
                        }
                    }
                    else {
                        propertyValueDump = JSON.stringify(object);
                    }
                }
                return objectTypeDump + ": " + propertyValueDump;
            }

            function _unwrapFunction(funcName) {
                return function (thisArg) {
                    var args = [];
                    for (var _i = 1; _i < arguments.length; _i++) {
                        args[_i - 1] = arguments[_i];
                    }
                    if ((thisArg || thisArg === EMPTY) && thisArg[funcName]) {
                        return thisArg[funcName].apply(thisArg, args);
                    }
                    throwTypeError("'" + funcName + "' not defined for " + dumpObj(thisArg));
                };
            }
            function _unwrapProp(propName) {
                return function (thisArg) {
                    return thisArg[propName];
                };
            }

            var arrIndexOf = exports('arrIndexOf', _unwrapFunction(INDEX_OF));

            var arrMap = exports('arrMap', _unwrapFunction("map"));

            var arrReduce = exports('arrReduce', _unwrapFunction("reduce"));

            function objSetPrototypeOf(obj, proto) {
                var fn = ObjClass["setPrototypeOf"] ||
                    ({ __proto__: [] } instanceof Array && function (d, b) {
                        d.__proto__ = b;
                    }) ||
                    function (d, b) {
                        objForEachKey(b, function (key, value) {
                            d[key] = value;
                        });
                    };
                return fn(obj, proto);
            }

            function _createCustomError(name, d, b) {
                objSetPrototypeOf(d, b);
                function __() {
                    this.constructor = d;
                }
                __[PROTOTYPE] = b[PROTOTYPE];
                __[PROTOTYPE].name = name;
                d[PROTOTYPE] = new __();
                return d;
            }
            function createCustomError(name, constructCb) {
                var baseClass = Error;
                var customError = _createCustomError(name, function () {
                    var _this = this;
                    _this = baseClass.apply(_this, arguments) || _this;
                    constructCb && constructCb(_this, arguments);
                    return _this;
                }, baseClass);
                return customError;
            }
            var _unsupportedError;
            function throwUnsupported(message) {
                if (!_unsupportedError) {
                    _unsupportedError = createCustomError("UnsupportedError");
                }
                throw new _unsupportedError(message);
            }

            function _doExtend(target, theArgs) {
                arrForEach(theArgs, function (theArg) {
                    objCopyProps(target, theArg);
                });
                return target;
            }
            function deepExtend(target) {
                var theArgs = [];
                for (var _i = 1; _i < arguments.length; _i++) {
                    theArgs[_i - 1] = arguments[_i];
                }
                return _doExtend(objDeepCopy(target) || {}, theArgs);
            }
            function objExtend(target) {
                var theArgs = [];
                for (var _i = 1; _i < arguments.length; _i++) {
                    theArgs[_i - 1] = arguments[_i];
                }
                return _doExtend(target || {}, theArgs);
            }

            var getLength = exports('getLength', _unwrapProp(LENGTH));

            function getIntValue(value, defValue) {
                if (!isNullOrUndefined(value)) {
                    if (isNumber(value)) {
                        return value;
                    }
                    var theValue = parseInt(value, 10);
                    return isNaN(theValue) ? defValue : theValue;
                }
                return defValue;
            }

            function _checkLength(value, props) {
                var result;
                arrForEach(props, function (prop) {
                    if (prop in value) {
                        var propValue = value[prop];
                        result = (isFunction(propValue) ? propValue() : propValue) > 0;
                        return -1;
                    }
                });
                return result;
            }
            function _hasValue(value, depth) {
                var result = value === false || value === 0;
                if (!result && !isNullOrUndefined(value)) {
                    if (isArray(value)) {
                        result = value[LENGTH] > 0;
                    }
                    else if (isDate(value)) {
                        result = !isNaN(value.getTime());
                    }
                    else if (isBoolean(value)) {
                        return true;
                    }
                    else if (isObject(value)) {
                        try {
                            var chkValue = _checkLength(value, [LENGTH, "byteLength", "size", "count"]);
                            if (isBoolean(chkValue)) {
                                return chkValue;
                            }
                            if (isFunction(value.valueOf) && depth < 5) {
                                return _hasValue(value.valueOf(), ++depth);
                            }
                        }
                        catch (e) {
                        }
                        return !!objKeys(value)[LENGTH];
                    }
                    else {
                        result = isTruthy(value);
                    }
                }
                return result;
            }
            function hasValue(value) {
                return _hasValue(value, 0);
            }

            var _objDefineProperty = ObjClass["defineProperty"];
            function objDefineProp(target, key, descriptor) {
                if (_objDefineProperty) {
                    _objDefineProperty(target, key, descriptor);
                    return target;
                }
                throwUnsupported("Unable to define property [" + objToString(key) + "] on " + dumpObj(target));
            }
            function objDefineGet(target, key, value, configurable) {
                var desc = {
                    enumerable: true,
                    configurable: isUndefined(configurable) ? true : configurable
                };
                if (isFunction(value)) {
                    desc.get = value;
                }
                else {
                    desc[VALUE] = value;
                }
                return objDefineProp(target, key, desc);
            }
            function objDefineAccessors(target, prop, getProp, setProp, configurable) {
                if (_objDefineProperty) {
                    try {
                        var descriptor = {
                            enumerable: true,
                            configurable: isUndefined(configurable) ? true : configurable
                        };
                        if (getProp) {
                            descriptor.get = getProp;
                        }
                        if (setProp) {
                            descriptor.set = setProp;
                        }
                        return objDefineProp(target, prop, descriptor);
                    }
                    catch (e) {
                        throwUnsupported("Unable to define accessors for [" + objToString(prop) + "] on " + dumpObj(target));
                    }
                }
            }

            function createIterable(ctx) {
                return makeIterable({}, ctx);
            }
            function makeIterable(target, ctx) {
                var itSymbol = getKnownSymbol(3);
                function _createIterator() {
                    return createIterator(ctx);
                }
                target[itSymbol] = _createIterator;
                return target;
            }
            function createIterator(ctx) {
                var isDone = false;
                function _value() {
                    return ctx.v;
                }
                function _next() {
                    var _a;
                    var args = [];
                    for (var _i = 0; _i < arguments.length; _i++) {
                        args[_i] = arguments[_i];
                    }
                    isDone = isDone || (ctx.n ? ctx.n(args) : true);
                    var result = (_a = {},
                        _a[DONE] = isDone,
                        _a);
                    if (!isDone) {
                        objDefineGet(result, VALUE, _value);
                    }
                    return result;
                }
                function _return(value) {
                    var _a;
                    return _a = {},
                        _a[DONE] = true,
                        _a[VALUE] = ctx.r && ctx.r(value),
                        _a;
                }
                function _throw(e) {
                    var _a;
                    return _a = {},
                        _a[DONE] = true,
                        _a[VALUE] = ctx.t && ctx.t(e),
                        _a;
                }
                var theIterator = {
                    next: _next
                };
                if (ctx.r) {
                    theIterator.return = _return;
                }
                if (ctx.t) {
                    theIterator.throw = _throw;
                }
                return theIterator;
            }

            function createArrayIterator(values) {
                var idx = -1;
                var theValues = values ? values.slice() : [];
                var len = theValues[LENGTH];
                function _value() {
                    if (idx >= 0 && idx < len) {
                        return theValues[idx];
                    }
                }
                function _getNext() {
                    idx++;
                    return idx >= len;
                }
                var ctx = {
                    n: _getNext
                };
                objDefineAccessors(ctx, "v", _value);
                return createIterator(ctx);
            }

            function iterForOf(iter, callbackfn, thisArg) {
                if (iter) {
                    if (!isIterator(iter)) {
                        var itSymbol = getKnownSymbol(3);
                        iter = iter[itSymbol] ? iter[itSymbol]() : null;
                    }
                    if (isIterator(iter)) {
                        try {
                            var count = 0;
                            var value = iter.next();
                            while (!value[DONE]) {
                                if (callbackfn.call(thisArg || iter, value[VALUE], count, iter) === -1) {
                                    break;
                                }
                                count++;
                                value = iter.next();
                            }
                            iter.return && iter.return(value);
                        }
                        catch (e) {
                            iter.throw && iter.throw(e);
                        }
                    }
                }
            }

            function createRangeIterator(start, end, step) {
                var nextValue = start;
                var theValue = UNDEF_VALUE;
                if (isNullOrUndefined(end)) {
                    end = start;
                }
                var theStep = step || ((start <= end) ? 1 : -1);
                function _value() {
                    return theValue;
                }
                function _getNext() {
                    var isDone = false;
                    if (theStep > 0) {
                        isDone = nextValue > end;
                    }
                    else {
                        isDone = nextValue < end;
                    }
                    if (!isDone) {
                        theValue = nextValue;
                        nextValue += theStep;
                    }
                    return isDone;
                }
                var ctx = {
                    n: _getNext
                };
                objDefineAccessors(ctx, "v", _value);
                return createIterator(ctx);
            }

            var mathFloor = exports('mathFloor', MathCls.floor);
            var mathCeil = exports('mathCeil', MathCls.ceil);

            var mathMin = exports('mathMin', MathCls.min);
            var mathMax = exports('mathMax', MathCls.max);

            var _mathTrunc = MathCls.trunc;
            var mathTrunc = exports('mathTrunc', _mathTrunc ? _mathTrunc : polyMathTrunc);
            function polyMathTrunc(value) {
                var theValue = +value;
                return (theValue > 0 ? mathFloor : mathCeil)(theValue);
            }

            function mathToInt(value, throwInfinity) {
                var result = +value;
                if (result == Infinity && throwInfinity) {
                    throwRangeError("invalid value [" + dumpObj(value) + "]");
                }
                return result !== result || result === 0 ? 0 : mathTrunc(result);
            }

            var _objCreate = ObjClass["create"];
            var objCreate = exports('objCreate', _objCreate ? _objCreate : polyObjCreate);
            function polyObjCreate(obj) {
                if (!obj) {
                    return {};
                }
                var type = typeof obj;
                if (type !== OBJECT && type !== FUNCTION) {
                    throw new TypeError("Prototype must be an Object or function: " + dumpObj(obj));
                }
                function tempFunc() { }
                tempFunc[PROTOTYPE] = obj;
                return new tempFunc();
            }

            var strSlice = exports('strSlice', _unwrapFunction("slice"));

            var SUB_STR = "substr";
            var strSubstring = exports('strSubstring', _unwrapFunction("substring"));
            var strSubstr = exports('strSubstr', StrProto[SUB_STR] ? _unwrapFunction(SUB_STR) : polyStrSubstr);
            function polyStrSubstr(value, start, length) {
                if (isNullOrUndefined(value)) {
                    throwTypeError("'polyStrSubstr called with invalid " + dumpObj(value));
                }
                if (length < 0) {
                    return EMPTY;
                }
                start = start || 0;
                if (start < 0) {
                    start = mathMax(start + value[LENGTH], 0);
                }
                if (isUndefined(length)) {
                    return strSlice(value, start);
                }
                return strSlice(value, start, start + length);
            }
            function strLeft(value, count) {
                return strSubstring(value, 0, count);
            }
            function strRight(value, count) {
                var len = value[LENGTH];
                if (count <= 0) {
                    return EMPTY;
                }
                return len > count ? strSubstring(value, len - count) : value;
            }

            var ENDS_WITH = "endsWith";
            var strEndsWith = exports('strEndsWith', StrProto[ENDS_WITH] ? _unwrapFunction(ENDS_WITH) : polyStrEndsWith);
            function polyStrEndsWith(value, searchString, length) {
                if (!isString(value)) {
                    throwTypeError("'" + dumpObj(value) + "' is not a string");
                }
                var searchValue = isString(searchString) ? searchString : "" + searchString;
                var chkLen = searchValue[LENGTH];
                var len = value[LENGTH];
                var end = !isUndefined(length) && length < len ? length : len;
                return strSubstring(value, end - chkLen, end) === searchValue;
            }

            var strIndexOf = exports('strIndexOf', _unwrapFunction(INDEX_OF));
            var strLastIndexOf = exports('strLastIndexOf', _unwrapFunction(LAST_INDEX_OF));

            function strIsNullOrWhiteSpace(value) {
                if (isString(value)) {
                    return value.replace(/[\s\t\r\n\f]+/g, EMPTY) === EMPTY;
                }
                return isNullOrUndefined(value);
            }
            function strIsNullOrEmpty(value) {
                if (isString(value)) {
                    return value === EMPTY;
                }
                return isNullOrUndefined(value);
            }

            var REPEAT = "repeat";
            var strRepeat = exports('strRepeat', StrProto[REPEAT] ? _unwrapFunction(REPEAT) : polyStrRepeat);
            function polyStrRepeat(value, count) {
                if (isNullOrUndefined(value)) {
                    throwTypeError("can't convert [" + dumpObj(value) + "]");
                }
                count = mathToInt(count, true);
                if (count < 0) {
                    throwRangeError("invalid count must be >= 0 && < Infinity");
                }
                var pad = isString(value) ? value : "" + value;
                var result = EMPTY;
                for (; count > 0; (count >>>= 1) && (pad += pad)) {
                    if (count & 1) {
                        result += pad;
                    }
                }
                return result;
            }

            var PAD_START = "padStart";
            var PAD_END = "padEnd";
            function _padValue(value, targetLength, padString) {
                var result = EMPTY;
                targetLength = mathToInt(targetLength, true);
                targetLength >>= 0;
                var len = value[LENGTH];
                if (len < targetLength) {
                    result = isNullOrUndefined(padString) ? " " : "" + padString;
                    targetLength = targetLength - len;
                    if (targetLength > result[LENGTH]) {
                        result = strRepeat(result, mathCeil(targetLength / result[LENGTH]));
                    }
                    if (result[LENGTH] > targetLength) {
                        result = strSubstring(result, 0, targetLength);
                    }
                }
                return result;
            }
            var strPadStart = exports('strPadStart', StrProto[PAD_START] ? _unwrapFunction(PAD_START) : polyStrPadStart);
            var strPadEnd = exports('strPadEnd', StrProto[PAD_END] ? _unwrapFunction(PAD_END) : polyStrPadEnd);
            function polyStrPadStart(value, targetLength, padString) {
                return _padValue(value, targetLength, padString) + value;
            }
            function polyStrPadEnd(value, targetLength, padString) {
                return value + _padValue(value, targetLength, padString);
            }

            var STARTS_WITH = "startsWith";
            var strStartsWith = exports('strStartsWith', StrProto[STARTS_WITH] ? _unwrapFunction(STARTS_WITH) : polyStrStartsWith);
            function polyStrStartsWith(value, searchString, position) {
                if (!isString(value)) {
                    throwTypeError("'" + dumpObj(value) + "' is not a string");
                }
                var searchValue = isString(searchString) ? searchString : "" + searchString;
                var chkLen = searchValue[LENGTH];
                var pos = position > 0 ? position : 0;
                return strSubstring(value, pos, pos + chkLen) === searchValue;
            }

            function _createTrimFn(exp) {
                return function _doTrim(value) {
                    if (isNullOrUndefined(value)) {
                        throwTypeError("strTrim called [" + dumpObj(value) + "]");
                    }
                    if (value && value.replace) {
                        value = value.replace(exp, "");
                    }
                    return value;
                };
            }
            var polyStrTrim = exports('polyStrTrim', _createTrimFn(/^\s+|(?=\s)\s+$/g));
            var polyStrTrimStart = exports('polyStrTrimStart', _createTrimFn(/^\s+/g));
            var polyStrTrimEnd = exports('polyStrTrimEnd', _createTrimFn(/(?=\s)\s+$/g));

            var TRIM = "trim";
            var TRIM_START = "trimStart";
            var TRIM_END = "trimEnd";
            var strTrim = exports('strTrim', StrProto[TRIM] ? _unwrapFunction(TRIM) : polyStrTrim);
            var strTrimStart = exports('strTrimStart', StrProto[TRIM_START] ? _unwrapFunction(TRIM_START) : polyStrTrimStart);
            var strTrimLeft = exports('strTrimLeft', strTrimStart);
            var strTrimEnd = exports('strTrimEnd', StrProto[TRIM_END] ? _unwrapFunction(TRIM_END) : polyStrTrimEnd);
            var strTrimRight = exports('strTrimRight', strTrimEnd);

            function polyIsArray(value) {
                if (isNullOrUndefined(value)) {
                    return false;
                }
                return objToString(value) === "[object Array]";
            }

            function polyObjKeys(obj) {
                if (!isObject(obj) || obj === null) {
                    throw new TypeError("polyObjKeys called on non-object");
                }
                var result = [];
                for (var prop in obj) {
                    if (objHasOwnProperty(obj, prop)) {
                        result.push(prop);
                    }
                }
                return result;
            }

        })
    };
}));
//# sourceMappingURL=ts-utils.js.map
